import path from 'path';
import url from 'url';

import esmock from './esmock.js';
export default esmock;

// ex, file:///path/to/esmock,
//     file:///c:/path/to/esmock
const urlDummy = 'file:///' + path
  .join(path.dirname(url.fileURLToPath(import.meta.url)), 'esmock.js')
  .replace(/\//, '');

export async function resolve (specifier, context, defaultResolve) {
  const [ esmockKeyParam ] = (context.parentURL
    && context.parentURL.match(/esmockKey=\d*/) || []);

  if (!esmockKeyParam)
    return defaultResolve(specifier, context, defaultResolve);

  const resolved = defaultResolve(specifier, context, defaultResolve);
  const moduleKeyRe = new RegExp(
    '.*(' + resolved.url + '\\?' + esmockKeyParam + '[^#]*).*');

  const moduleURLSplitKeys = context.parentURL.split('#esmockModuleKeys=');
  // eslint-disable-next-line prefer-destructuring
  const moduleGlobals = moduleURLSplitKeys[0].split('?esmockGlobals=')[1];
  const moduleKeyChild = moduleKeyRe.test(moduleURLSplitKeys[1])
        && moduleURLSplitKeys[1].replace(moduleKeyRe, '$1');
  const moduleKeyGlobal = moduleKeyRe.test(moduleGlobals)
        && moduleGlobals.replace(moduleKeyRe, '$1');

  const moduleKey = moduleKeyChild || moduleKeyGlobal;
  if (moduleKey) {
    resolved.url = /isesm=true/.test(moduleKey)
      ? moduleKey
      : urlDummy + '#' + moduleKey;
  } else if (moduleGlobals && moduleGlobals !== 'null') {
    if (!resolved.url.startsWith('node:')) {
      resolved.url += '?esmockGlobals=' + moduleGlobals;
    }
  }

  return resolved;
}

export async function load (url, context, defaultGetSource) {
  if (/#esmockModuleKeys/gi.test(url)) // parent of mocked modules
    return defaultGetSource(url, context, defaultGetSource);

  [ url ] = url.split('?esmockGlobals=');
  if (url.startsWith(urlDummy)) {
    url = url.replace(/[^#]*#/, '');
  }

  const exportedNames = /exportNames=/.test(url) &&
    url.replace(/.*exportNames=(.*)/, '$1').split(',');
  if (exportedNames) {
    return {
      format : 'module',
      source : exportedNames.map(name => name === 'default'
        ? `export default global.esmockCacheGet("${url}").default`
        : `export const ${name} = global.esmockCacheGet("${url}").${name}`
      ).join('\n')
    };
  }

  return defaultGetSource(url, context, defaultGetSource);
}

// supported by node version less than 16.12
const [ nodeMjr, nodeMnr ] = process.versions.node.split('.').map(Number);
const nodelt1612 = nodeMjr < 16 || (nodeMjr === 16 && nodeMnr < 12);

async function getSource (url, context, defaultGetSource) {
  return load(url, context, defaultGetSource);
}

// make this null node versions after 16.11 so taht warning message
// is not printed
if (!nodelt1612) getSource = null;

export { getSource };
